//---------------------------------------------------------------------
#property copyright 	"Dima S."
#property link      	"dimascub@mail.com"
#property version   	"1.00"
#property description " ."
//---------------------------------------------------------------------

//---------------------------------------------------------------------
//	 :
//---------------------------------------------------------------------
#include	<Trade\Trade.mqh>
#include	<BalanceSlopeControl.mqh>
//---------------------------------------------------------------------

//=====================================================================
//	  :
//=====================================================================

//---------------------------------------------------------------------
//	     ;
//---------------------------------------------------------------------
enum SetLogic 
{
	No = 0,
	Yes = 1,
};
//---------------------------------------------------------------------
input SetLogic		UseAutoBalanceControl = No;
//---------------------------------------------------------------------
input ControlType	BalanceControlType = STEP_WITHOUT_HYSTERESIS;
//---------------------------------------------------------------------
//	       :
input int					TradesNumberToCalcLR = 3;
//---------------------------------------------------------------------
//	        :
input double			LRKoeffForRejectLots = -0.020;
//---------------------------------------------------------------------
//	       :
input double			LRKoeffForRestoreLots = 0.005;
//---------------------------------------------------------------------
//	      :
input double			LRKoeffForIntermedLots = -0.019;
//---------------------------------------------------------------------
//	         
input double			RejectedLots = 0.10;
//---------------------------------------------------------------------
//	        :
input double			NormalLots = 1.0;
//---------------------------------------------------------------------
input int									CCIPeriod = 50;
input ENUM_APPLIED_PRICE	CCIPrice = PRICE_TYPICAL;
//---------------------------------------------------------------------

//---------------------------------------------------------------------
int			ts1_indicator_handle1 = 0;
//---------------------------------------------------------------------

TBalanceSlopeControl	BalanceControl;

//---------------------------------------------------------------------
//	  :
//---------------------------------------------------------------------
int
OnInit( )
{
//	        :
	ResetLastError( );
	ts1_indicator_handle1 = iCCI( Symbol( ), PERIOD_CURRENT, CCIPeriod, CCIPrice );

//	   ,    :
	if( ts1_indicator_handle1 == INVALID_HANDLE )
	{
		Print( "   CCI,  = ", GetLastError( ));
		return( -1 );
	}

//	     :
	BalanceControl.SetTradeSymbol( Symbol( ));
	BalanceControl.SetControlType( BalanceControlType );
	BalanceControl.SetControlParams( LRKoeffForRejectLots, LRKoeffForRestoreLots, LRKoeffForIntermedLots );
	BalanceControl.SetSlopePoints( TradesNumberToCalcLR );
	BalanceControl.SetFiltrParams( 0, -1, 0 );
	BalanceControl.SetMonitoringBeginDate( 0 );

	return( 0 );
}

//---------------------------------------------------------------------
//	  -:
//---------------------------------------------------------------------
void
OnDeinit( const int _reason )
{
//	  :
	if( ts1_indicator_handle1 != INVALID_HANDLE )
	{
		IndicatorRelease( ts1_indicator_handle1 );
	}
}

//---------------------------------------------------------------------
//	        :
//---------------------------------------------------------------------
int			current_signal = 0;
double	current_lots;
//---------------------------------------------------------------------
void
OnTick( )
{
//	   :
	if( CheakNewBar( ) != 1 )
	{
		return;
	}

//	   / :
	current_signal = GetSignal( 1 );

//	    :
	if( PositionSelect( Symbol( )) == true )
	{
//	,      :
		if( CheakPositionClose( current_signal ) == 1 )
		{
			return;
		}
	}

//	  :
	BalanceControl.RefreshSymbolInfo( );

//	    BUY:
	if( CheckBuySignal( current_signal ) == 1 )
	{
		if( UseAutoBalanceControl == Yes )
		{
			current_lots = BalanceControl.CalcTradeLots( RejectedLots, NormalLots );
		}
		else
		{
			current_lots = NormalLots;
		}
		CTrade	trade;
		trade.PositionOpen( Symbol( ), ORDER_TYPE_BUY, current_lots, SymbolInfoDouble( Symbol( ), SYMBOL_ASK ), 0, 0 );
	}

//	    SELL:
	if( CheckSellSignal( current_signal ) == 1 )
	{
		if( UseAutoBalanceControl == Yes )
		{
			current_lots = BalanceControl.CalcTradeLots( RejectedLots, NormalLots );
		}
		else
		{
			current_lots = NormalLots;
		}
		CTrade	trade;
		trade.PositionOpen( Symbol( ), ORDER_TYPE_SELL, current_lots, SymbolInfoDouble( Symbol( ), SYMBOL_BID ), 0, 0 );
	}
}

//---------------------------------------------------------------------
//	,     :
//---------------------------------------------------------------------
//	:
//		0 -   ;
//		1 -      ;
//---------------------------------------------------------------------
int
CheakPositionClose( int _signal )
{
	long		position_type = PositionGetInteger( POSITION_TYPE );

	if( _signal == 1 )
	{
//	    BUY,  :
		if( position_type == ( long )POSITION_TYPE_BUY )
		{
			return( 1 );
		}
	}

	if( _signal == -1 )
	{
//	    SELL,  :
		if( position_type == ( long )POSITION_TYPE_SELL )
		{
			return( 1 );
		}
	}

//	 :
	CTrade	trade;
	trade.PositionClose( Symbol( ), 10 );

	return( 0 );
}

//---------------------------------------------------------------------
//	    BUY:
//---------------------------------------------------------------------
//	:
//		0 -  ;
//		1 -    BUY;
//---------------------------------------------------------------------
int
CheckBuySignal( int _curr_signal )
{
	if( _curr_signal == 1 )
	{
		return( 1 );
	}

	return( 0 );
}

//---------------------------------------------------------------------
//	    SELL:
//---------------------------------------------------------------------
//	:
//		0 -  ;
//		1 -    SELL;
//---------------------------------------------------------------------
int
CheckSellSignal( int _curr_signal )
{
	if( _curr_signal == -1 )
	{
		return( 1 );
	}

	return( 0 );
}

//---------------------------------------------------------------------
//	   / :
//---------------------------------------------------------------------
int
GetSignal( int _index )
{
	double		cci_array[ 2 ];

//	  CCI:
	ResetLastError( );
	if( CopyBuffer( ts1_indicator_handle1, 0, _index, 2, cci_array ) != 2 )
	{
		Print( "  CopyBuffer,  = ", GetLastError( ));
		return( 0 );
	}

	if( cci_array[ 1 ] >= 45.0 && cci_array[ 0 ] < 0.0 )
	{
		return( 1 );
	}
	else if( cci_array[ 1 ] <= 45.0 && cci_array[ 0 ] > 0.0 )
	{
		return( -1 );
	}

	return( 0 );
}

//---------------------------------------------------------------------
//	    :
//---------------------------------------------------------------------
//	-   1,   	;
//---------------------------------------------------------------------
int
CheakNewBar( )
{
	MqlRates		current_rates[ 1 ];

	ResetLastError( );
	if( CopyRates( Symbol( ), Period( ), 0, 1, current_rates ) != 1 )
	{
		Print( "  CopyRates,  = ", GetLastError( ));
		return( 0 );
	}

	if( current_rates[ 0 ].tick_volume > 1 )
	{
		return( 0 );
	}

	return( 1 );
}
